## $Id: browser.py 18046 2025-03-15 23:59:29Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from kofacustom.nigeria.interswitch.browser import (
    module_activated,
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPaymentVerifyWebservicePageApplicant,
    InterswitchPaymentVerifyWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from kofacustom.unidel.students.interfaces import ICustomStudentOnlinePayment
from kofacustom.unidel.applicants.interfaces import ICustomApplicantOnlinePayment
from kofacustom.unidel.interfaces import MessageFactory as _

PRODUCT_ID = '22153701' # must be provided by Interswitch
SITE_NAME = 'unidel.waeup.org'
PROVIDER_ACCT = '0213065415'
PROVIDER_BANK_ID = '47'
PROVIDER_ITEM_NAME = 'WAeAC'
INSTITUTION_NAME = 'UNIDEL'
CURRENCY = '566'
GATEWAY_AMT = 300.0
#MAC = 'CEF793CBBE838AA0CBB29B74D571113B4EA6586D3BA77E7CFA0B95E278364EFC4526ED7BD255A366CDDE11F1F607F0F844B09D93B16F7CFE87563B2272007AB3'
MAC = 'Lfj58PyWS6MvPz65zIbofZNoAn89fZAjtnsEWbTof73OyHQH7rpJPHWD2m4cekDoowJ8nqQCn6ay2lopzKBOekFsMfzkXG6KYGRuyhMQq4bK7wDNaWqpxeZl3fMumNmi'

POST_ACTION = 'https://webpay.interswitchng.com/collections/w/pay'
#POST_ACTION = 'https://sandbox.interswitchng.com/webpay/pay'
HOST = 'webpay.interswitchng.com'
#HOST = 'sandbox.interswitchng.com'
URL = '/collections/api/v1/gettransaction.json'
#URL = '/webpay/api/v1/gettransaction.json'

httplib.HTTPSConnection.debuglevel = 0
HTTPS = True

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = MAC
    gateway_amt = GATEWAY_AMT

    def update(self):
        if not module_activated(
            self.context.student.current_session, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_acct'] = '000000000'
        xmldict['institution_bank_id'] = '117'
        provider_amt = 0.0
        tech_fee = 0.0
        union_due = 0.0
        if self.context.p_category == 'clearance':
            provider_amt = 2000.0
            if student.current_mode in ('ug_ft', 'de_ft'):
                xmldict['institution_acct'] = '1216063205'
                tech_fee = 18000.0
                xmldict['tech_acct'] = '0092304647'
                xmldict['tech_bank_id'] = '121'
            elif student.current_mode.startswith('dp'):
                xmldict['institution_acct'] = '1011431799'
            if student.faccode in ('PRE', 'JUPEB'):
                xmldict['institution_acct'] = '1011431799'
            elif student.faccode == 'IJMB':
                xmldict['institution_acct'] = '0093658062'
                xmldict['institution_bank_id'] = '121'
            if self.context.p_item == 'Balance':
                provider_amt = 0.0
                tech_fee = 0.0
        elif self.context.p_category == 'schoolfee':
            provider_amt = 7000.0
            tech_fee = 3000.0
            xmldict['institution_acct'] = '1011739172'
            xmldict['tech_acct'] = '0213065415'
            xmldict['tech_bank_id'] = '47'
            xmldict['union_acct'] = '1011438901'
            xmldict['union_bank_id'] = '117'
            if student.current_mode == 'ug_ft':
                union_due = 1500.0
            if student.current_mode.startswith('dp'):
                xmldict['institution_acct'] = '2001627961'
                xmldict['institution_bank_id'] = '8'
            elif student.current_mode in ('pde', 'trc'):
                xmldict['institution_acct'] = '1010957205'
                xmldict['institution_bank_id'] = '117'
            elif student.faccode in ('PRE', 'JUPEB'):
                xmldict['institution_acct'] = '2001627961'
                xmldict['institution_bank_id'] = '8'
            elif student.faccode == 'DELSU':
                xmldict['institution_acct'] = '1011036493'
            elif student.faccode == 'NCE':
                xmldict['institution_acct'] = '1011274462'
                provider_amt = 4000.0
                tech_fee = 0.0
            elif student.faccode == 'IJMB':
                xmldict['institution_acct'] = '0093658062'
                xmldict['institution_bank_id'] = '121'
            elif student.faccode in ('FMED', 'FBM', 'FLW'):
                xmldict['institution_acct'] = '1011274462'
            elif student.certcode == 'DFFPA':
                xmldict['institution_acct'] = '2001627961'
                xmldict['institution_bank_id'] = '8'
                provider_amt = 4000.0
                tech_fee = 0.0
            if self.context.p_item == 'Balance':
                provider_amt = 0.0
                tech_fee = 0.0
                union_due = 0.0
        elif self.context.p_category.startswith('ijmb'):
                xmldict['institution_acct'] = '0093658062'
                xmldict['institution_bank_id'] = '121'
        elif self.context.p_category == 'hostel_maintenance':
            xmldict['institution_acct'] = '1012551384'
        elif self.context.p_category == 'med_reg':
            xmldict['institution_acct'] = '1011265606'
        elif self.context.p_category == 'teaching_practice':
            xmldict['institution_acct'] = '1005399320'
            xmldict['institution_bank_id'] = '8'
        elif self.context.p_category == 'siwes':
            xmldict['institution_acct'] = '2000092025'
            xmldict['institution_bank_id'] = '8'
        elif self.context.p_category.startswith('transcript'):
            xmldict['institution_acct'] = '0010494552'
            xmldict['institution_bank_id'] = '129'
            provider_amt = 2000.0
        elif self.context.p_category.startswith('union'):
            xmldict['institution_acct'] = '1011438901'
        self.pay_item_id = 'Default_Payable_MX47126' # must be provided by Interswitch
        # Already now it becomes an Interswitch payment. We set the net amount
        # and add gateway amount and other surcharges.
        if not self.context.r_company:
            self.context.r_company = u'interswitch'
            self.context.net_amt = self.context.amount_auth
            self.context.gateway_amt = self.gateway_amt
            self.context.amount_auth += self.gateway_amt
            self.context.provider_amt = provider_amt
            self.context.amount_auth += provider_amt
            self.context.amount_auth += tech_fee
            self.context.amount_auth += union_due
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['tech_fee'] = 100 * tech_fee
        xmldict['union_due'] = 100 * union_due
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * self.context.net_amt
        xmldict['item_id'] = 1
        xmltext = """
<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="%(item_id)s" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
""" % xmldict
        if provider_amt:
            xmldict['item_id'] += 1
            xmltext += """<item_detail item_id="%(item_id)s" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
""" % xmldict
        if tech_fee:
            xmldict['item_id'] += 1
            xmltext += """<item_detail item_id="%(item_id)s" item_name="Technology Fee" item_amt="%(tech_fee)d" bank_id="%(tech_bank_id)s" acct_num="%(tech_acct)s" />
""" % xmldict
        if union_due:
            xmldict['item_id'] += 1
            xmltext += """<item_detail item_id="%(item_id)s" item_name="Student Union Due" item_amt="%(union_due)d" bank_id="%(union_bank_id)s" acct_num="%(union_acct)s" />
""" % xmldict
        xmltext +="""</item_details>
</payment_item_detail>
"""
        self.xml_data = """<input type="hidden" name="xml_data" value='%s' />""" % xmltext
        self.context.provider_amt = provider_amt
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = 'Default_Payable_MX47126' # must be provided by Interswitch
    product_id = PRODUCT_ID
    mac = MAC
    gateway_amt = GATEWAY_AMT

    def update(self):
        if not module_activated(
            self.context.__parent__.__parent__.year, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        # Already now it becomes an Interswitch payment. We set the net amount
        # and add the gateway amount.
        if not self.context.r_company:
            self.context.net_amt = self.context.amount_auth
            self.context.amount_auth += self.gateway_amt
            self.context.gateway_amt = self.gateway_amt
            self.context.r_company = u'interswitch'
        xmldict = {}
        provider_amt = 250.0
        xmldict['institution_acct'] = '1216063205'
        xmldict['institution_bank_id'] = '117'
        if self.context.__parent__.__parent__.prefix in (
                                    'pre', 'jupeb', 'dpft', 'dppt',
                                    'ugpt', 'ijmb', 'df'):
            xmldict['institution_acct'] = '1011431799'
            xmldict['institution_bank_id'] = '117'
            provider_amt = 500.0           
        if self.context.__parent__.__parent__.prefix in ('ijmb',):
            xmldict['institution_acct'] = '0093658062'
            xmldict['institution_bank_id'] = '121'
        if self.context.__parent__.__parent__.prefix in ('pde', 'trcdeg', 'trcnce'):
            xmldict['institution_acct'] = '1010957205'
            xmldict['institution_bank_id'] = '117'
            provider_amt = 500.0
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * self.context.net_amt
        if not self.context.provider_amt:
            self.context.provider_amt = provider_amt
            self.context.amount_auth += provider_amt
        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageStudent(
    InterswitchPaymentVerifyWebservicePageStudent):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageApplicant(
    InterswitchPaymentVerifyWebservicePageApplicant):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

# PAYDirect

from kofacustom.nigeria.interswitch.paydirectbrowser import (
    PAYDirectPageStudent, PAYDirectPageApplicant,
    StudentRefNumberSlip, ApplicantRefNumberSlip,
    )

from kofacustom.nigeria.interswitch.tests import (
    PAYDIRECT_URL, PAYDIRECT_HOST, MERCHANT_ID)

#PAYDIRECT_HOST = 'orion.interswitchng.com'
#PAYDIRECT_URL = '/bookonhold/bookonhold.asmx'
#MERCHANT_ID = ''

class CustomPAYDirectPageStudent(PAYDirectPageStudent):
    """Inform student how to proceed
    """
    grok.context(ICustomStudentOnlinePayment)
    gateway_amt = GATEWAY_AMT
    merchant_id = MERCHANT_ID
    gateway_url = PAYDIRECT_URL
    gateway_host = PAYDIRECT_HOST
    https = True

class CustomPAYDirectPageApplicant(PAYDirectPageApplicant):
    """ Inform applicant how to proceed.
    """
    grok.context(ICustomApplicantOnlinePayment)
    gateway_amt = GATEWAY_AMT
    merchant_id = MERCHANT_ID
    gateway_url = PAYDIRECT_URL
    gateway_host = PAYDIRECT_HOST
    https = True

class CustomStudentRefNumberSlip(StudentRefNumberSlip):
    """Deliver a PDF slip of the context.
    """
    merchant_id = MERCHANT_ID

class CustomApplicantRefNumberSlip(ApplicantRefNumberSlip):
    """Deliver a PDF slip of the context.
    """
    merchant_id = MERCHANT_ID
